@@ -17,9 +17,11 @@
  */
 package org.apache.cassandra.db;
 
+import java.io.IOException;
 import java.util.Collection;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ExecutionException;
 
 import javax.management.openmbean.CompositeData;
@@ -49,12 +51,7 @@ public interface ColumnFamilyStoreMBean
     public void forceMajorCompaction(boolean splitOutput) throws ExecutionException, InterruptedException;
 
     /**
-     * Forces a major compaction of specified token ranges in this column family.
-     * <p>
-     * The token ranges will be interpreted as closed intervals to match the closed interval defined by the first and
-     * last keys of a sstable, even though the {@link Range} class is suppossed to be half-open by definition.
-     *
-     * @param tokenRanges The token ranges to be compacted, interpreted as closed intervals.
+     * force a major compaction of specified key range in this column family
      */
     public void forceCompactionForTokenRange(Collection<Range<Token>> tokenRanges) throws ExecutionException, InterruptedException;
     /**
@@ -106,14 +103,15 @@ public interface ColumnFamilyStoreMBean
      * Get the compression parameters
      */
     public Map<String,String> getCompressionParameters();
+
     public String getCompressionParametersJson();
 
     /**
-     * Set the compression parameters
+     * Set the compression parameters locally for this node
      * @param opts map of string names to values
      */
     public void setCompressionParameters(Map<String,String> opts);
-    public void setCompressionParametersJson(String opts);
+    public void setCompressionParametersJson(String options);
 
     /**
      * Set new crc check chance
@@ -147,11 +145,52 @@ public interface ColumnFamilyStoreMBean
     public List<String> getSSTablesForKey(String key, boolean hexFormat);
 
     /**
-     * Scan through Keyspace/ColumnFamily's data directory
-     * determine which SSTables should be loaded and load them
+     * Load new sstables from the given directory
+     *
+     * @param srcPaths the path to the new sstables - if it is an empty set, the data directories will be scanned
+     * @param resetLevel if the level should be reset to 0 on the new sstables
+     * @param clearRepaired if repaired info should be wiped from the new sstables
+     * @param verifySSTables if the new sstables should be verified that they are not corrupt
+     * @param verifyTokens if the tokens in the new sstables should be verified that they are owned by the current node
+     * @param invalidateCaches if row cache should be invalidated for the keys in the new sstables
+     * @param extendedVerify if we should run an extended verify checking all values in the new sstables
+     *
+     * @return list of failed import directories
      */
-    public void loadNewSSTables();
+    @Deprecated
+    public List<String> importNewSSTables(Set<String> srcPaths,
+                                           boolean resetLevel,
+                                           boolean clearRepaired,
+                                           boolean verifySSTables,
+                                           boolean verifyTokens,
+                                           boolean invalidateCaches,
+                                           boolean extendedVerify);
 
+    /**
+     * Load new sstables from the given directory
+     *
+     * @param srcPaths the path to the new sstables - if it is an empty set, the data directories will be scanned
+     * @param resetLevel if the level should be reset to 0 on the new sstables
+     * @param clearRepaired if repaired info should be wiped from the new sstables
+     * @param verifySSTables if the new sstables should be verified that they are not corrupt
+     * @param verifyTokens if the tokens in the new sstables should be verified that they are owned by the current node
+     * @param invalidateCaches if row cache should be invalidated for the keys in the new sstables
+     * @param extendedVerify if we should run an extended verify checking all values in the new sstables
+     * @param copyData if we should copy data from source paths instead of moving them
+     *
+     * @return list of failed import directories
+     */
+    public List<String> importNewSSTables(Set<String> srcPaths,
+                                          boolean resetLevel,
+                                          boolean clearRepaired,
+                                          boolean verifySSTables,
+                                          boolean verifyTokens,
+                                          boolean invalidateCaches,
+                                          boolean extendedVerify,
+                                          boolean copyData);
+
+    @Deprecated
+    public void loadNewSSTables();
     /**
      * @return the number of SSTables in L0.  Always return 0 if Leveled compaction is not enabled.
      */
@@ -183,12 +222,12 @@ public interface ColumnFamilyStoreMBean
      * begin sampling for a specific sampler with a given capacity.  The cardinality may
      * be larger than the capacity, but depending on the use case it may affect its accuracy
      */
-    public void beginLocalSampling(String sampler, int capacity);
+    public void beginLocalSampling(String sampler, int capacity, int durationMillis);
 
     /**
      * @return top <i>count</i> items for the sampler since beginLocalSampling was called
      */
-    public CompositeData finishLocalSampling(String sampler, int count) throws OpenDataException;
+    public List<CompositeData> finishLocalSampling(String sampler, int count) throws OpenDataException;
 
     /*
         Is Compaction space check enabled
@@ -199,4 +238,17 @@ public interface ColumnFamilyStoreMBean
        Enable/Disable compaction space check
      */
     public void compactionDiskSpaceCheck(boolean enable);
+
+    public void setNeverPurgeTombstones(boolean value);
+
+    public boolean getNeverPurgeTombstones();
+
+    /**
+     * Check SSTables whether or not they are misplaced.
+     * @return true if any of the SSTables is misplaced.
+     *         If all SSTables are correctly placed or the partitioner does not support splitting, it returns false.
+     */
+    public boolean hasMisplacedSSTables();
+
+    public List<String> getDataPaths() throws IOException;
 }
